package tournament;

import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

public class TippManager extends MouseAdapter {
	
	private int dimension = 0;
	private int maxState = 1;

	private Vector<Tipp> leftTipps;
	private Vector<Tipp> rightTipps;
	private HashMap<String, Tipp> allTipps;
	
	private Cursor handCursor = new Cursor(Cursor.HAND_CURSOR);
	
	/**
	 * creates a new tippmanager with a list of players
	 * @param players the players used for this manager
	 */
	public TippManager(List<String> players)
	{
		this.dimension = players.size() / 2;
		
		leftTipps = new Vector<Tipp>();
		rightTipps = new Vector<Tipp>();
		allTipps = new HashMap<String, Tipp>();
		
		if (players.size() < dimension) 
			System.err.println ("TippManager::TippManager() not enough playernames");

		maxState = getMaxState(dimension);
		
		for (int i=0; i < dimension; i++)
		{
			Tipp tipp = new Tipp(20, 20.f+50*i, 94, 15, maxState);
			leftTipps.add(tipp);
			allTipps.put(players.get(i), tipp);
			tipp = new Tipp(800-20, 20.f+50*i, -94, 15, maxState);
			rightTipps.add(tipp);
			allTipps.put(players.get(i+dimension), tipp);
		}
		
	}
	
	/**
	 * sets the size of this tippmanager. this is called repeatedly when canvas
	 * is resized.
	 * @param x starting x of left tipps
	 * @param xEnd starting x of right tipps
	 * @param yStart y coord of the first tipp 
	 * @param yStep1 y distance between two opponents
	 * @param yStep2 y distance between two games
	 * @param length the length of one bracket
	 * @param thickness the thickness of the tippline
	 */
	public void setSize(int x, int xEnd, int yStart, int yStep1, int yStep2, int length, int thickness)
	{
		int y = yStart;
		for (int i=0; i < dimension; i++)
		{
			leftTipps.get(i).setTippDimensions(x, y, length, thickness);
			rightTipps.get(i).setTippDimensions(xEnd, y, -length, thickness);
			y += (i%2==0)?yStep1:yStep2;
		}
	}
	
	
	@Override
	public void mousePressed(MouseEvent e)
	{
		if (!processMouseClick(e, leftTipps, rightTipps))
			processMouseClick(e, rightTipps, leftTipps);
	}
	
	private boolean processMouseClick(MouseEvent e, List<Tipp> thisSide, List<Tipp> otherSide)
	{
		for (Tipp tipp:thisSide)
		{
			if (tipp.containsPoint(e.getX(), e.getY()))
			{
				if (e.getButton()==MouseEvent.BUTTON1)
				{
					int state = tipp.moveTippStateUp();
					validate(tipp, thisSide, state);
					
				}
				if (e.getButton()==MouseEvent.BUTTON3)
				{
					tipp.moveTippStateDown();
				}
				e.getComponent().repaint();
				return true;
			}
		}
		return false;
	}
	
	/**
	 * checks whether the current MouseEvent falls within the hotspot region of a
	 * tipp.
	 * @param e the mouseevent to check against
	 * @return true if the cursor is over a tippline<br>false otherwise
	 */
	public boolean containsCursor(MouseEvent e)
	{
		for (Tipp tipp:leftTipps)
		{
			if (tipp.containsPoint(e.getX(), e.getY()))
			{
				e.getComponent().setCursor(handCursor);
				return true;
			}
		}
		for (Tipp tipp:rightTipps)
		{
			if (tipp.containsPoint(e.getX(), e.getY()))
			{
				e.getComponent().setCursor(handCursor);
				return true;
			}
		}

		return false;
	}
	
	private void validate (Tipp tipp, List<Tipp> tippList, int state)
	{
		if (state == maxState) //if state is max (we have a winner) validate the opposite tree
		{
			List<Tipp> otherSide = tippList.equals(leftTipps)?rightTipps:leftTipps;
			for (Tipp t:otherSide)
			{
				if (t.getTippState()==maxState)
					t.moveTippStateDown();
			}
			return;
		}
		
		
		int range = (int)Math.pow(2, state); //range of elements to check
		int index = tippList.indexOf(tipp); 
		int myplace = index % range; //find my index in current range

		int subTree = index - myplace;
		
		Vector <Tipp> tipps = new Vector<Tipp>();
		
		for (int i = subTree; i < subTree+range; i++) //collect all elements within my subtree, with at least my state
		{
			Tipp ti = tippList.get(i); 
			
			if ((!ti.equals(tipp)) && (ti.getTippState()>=state)) //all but me
				tipps.add(ti);
		}
		
		if (tipps.size() >= 1)
		{
			for (int i = 0; i < tipps.size(); i++)
			{
				tipps.get(i).setTippState(state-1);
			}
			
		}
	}
	
	/**
	 * Returns the maximum tippline length (in segments). this equals the number of 
	 * games a team has to play to win the whole tournament.
	 * @param dimension number of players in the tournament
	 * @return maximum tippline length in segments.
	 */
	public int getMaxState(int dimension)
	{
		int i = 1;
		while (Math.pow(2,i) < dimension)
			i++;
		
		return i+1;		
	}


	/**
	 * paints all tipplines
	 * @param g2 the Graphics object to paint onto.
	 */
	public void paint(Graphics2D g2)
	{
		
		updateTippStates();
		
		for (Tipp tipp:leftTipps)
		{
			tipp.paint(g2);
		}	
		for (Tipp tipp:rightTipps)
		{
			tipp.paint(g2);
		}	
	}
	
	/**
	 * updates all the tipps states.
	 */
	public void updateTippStates()
	{
		for (Tipp tipp: allTipps.values())
		{
			tipp.setRealState(0);
			tipp.setTeamEliminated(false);
		}
		
		Tipp tipp = null;
		
		for (Game game : Game.allGames.values())
		{
			String team = game.getName();
			if ((tipp = allTipps.get(team))!=null)
			{
				tipp.moveRealStateUp();
				
				team = game.getNameInversed();
				if ((tipp = allTipps.get(team))!=null)
					tipp.setTeamEliminated(true); //TODO all to false first!"!
			}
		}
		
//		Game[] games = new Game[Game.allGames.size()]; 
//		Game.allGames.values().toArray(games);
//		Arrays.sort(games);
//		
//		System.out.println(games.length);
		
		
	}


	

	
}
