package tournament.xml;

import java.io.*;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Vector;

import org.xml.sax.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;

public class XMLParser {
	
	protected String fileName;
	protected String filePath;
	protected Vector<Team> teams;
	protected Vector<GameResult> games;
	protected Vector<Date> dates;
	
	protected HashMap<Integer, GameResult> resultMap;
	protected HashMap<Date, GameResult> resultDateMap;
	
	/**
	 * creates a new xmlparser with the default filename (tournament.xml)
	 */
	public XMLParser()
	{
		this("tournament.xml");	
	}
	
	/**
	 * creates a new xmlparser from specified file
	 * @param fileName the path to the xml file
	 */
	public XMLParser(String fileName)
	{
		setFileName(fileName);
		
		teams = new Vector<Team>();
		games = new Vector<GameResult>();
		dates = new Vector<Date>();
		
		resultMap = new HashMap<Integer, GameResult>();
		resultDateMap = new HashMap<Date, GameResult>();
	}
	
	/**
	 * clears all XML data.
	 */
	public void clear()
	{
		teams.clear();
		games.clear();
		dates.clear();
		resultMap.clear();
		resultDateMap.clear();
	}
	
	/**
	 * Parses and validates the xml file.
	 * @return true, if parsing and validating was successful<br>false, otherwise
	 */
	public boolean parse()
	{
		try {
			DocumentBuilder builder  = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			Document document = builder.parse( new File( fileName ) );
			NodeList teamList = document.getElementsByTagName( "team" );
			NodeList gameList = document.getElementsByTagName( "game" );
			
//			teams.clear();
//			games.clear();
			clear();
			
			for( int i=0; i<teamList.getLength(); i++ )
			{
				Element e = (Element)teamList.item(i);
				String name = e.getElementsByTagName("name").item(0).getTextContent();
				String desc = e.getElementsByTagName("descr").item(0).getTextContent();
				String icon = e.getElementsByTagName("icon").item(0).getTextContent();
				
				teams.add(new Team (name, desc, filePath+icon));
			}
			
			for( int i=0; i<gameList.getLength(); i++ )
			{
				String id = gameList.item(i).getAttributes().getNamedItem("id").getTextContent();
				
				Element e = (Element)gameList.item(i);
				String resl = e.getElementsByTagName("result").item(0).getTextContent();
				String desc = e.getElementsByTagName("descr").item(0).getTextContent();
				String date = e.getElementsByTagName("date").item(0).getTextContent();
				
				games.add(new GameResult (resl, id, desc, date));
			}
			
		} catch( SAXParseException spe ) {
			System.out.printf("\n** Parsing error: %s:%d", spe.getSystemId(), spe.getLineNumber());

			Exception e = ( spe.getException() != null ) ? spe.getException() : spe;
			e.printStackTrace();
			return false;
			
		} catch( Exception e)
		{
			e.printStackTrace();
			return false;
		}

		return verify();
	}
	
	private boolean verify()
	{
		int i = 1;
		int size = teams.size();
		
		while (Math.pow(2,i) < size)
			i++;
		
		boolean retval = Math.pow(2,i) == size;		
		
		for (GameResult game:games)
		{
			if (game.isValid() && (game.getGameId() < size))
			{
				resultMap.put(game.getGameId(), game);
//				System.out.println(game.getGameId() + " " + game);
				resultDateMap.put(game.getDate(), game);
				dates.add(game.getDate());
			}
		}
		Collections.sort(dates);

		return retval;
	}

	/**
	 * sets a new filename for this xmlparser. always call clear() before reparsing.  
	 * @param fileName path to xmlfile
	 */
	public void setFileName(String fileName)
	{
		this.fileName = fileName;
		this.filePath = "";
		
		if (fileName.contains("\\"))
			filePath = fileName.substring(0, fileName.lastIndexOf("\\")+1);
		
		if (fileName.contains("/"))
			filePath = fileName.substring(0, fileName.lastIndexOf("/")+1);
	}
	

}
